<?php

namespace Survey\Question\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Survey\Question\Http\Requests\DimensionRequest;
use Survey\Question\Interfaces\DimensionRepositoryInterface;
use Survey\Question\Models\Dimension;

/**
 * Resource controller class for dimension.
 */
class DimensionResourceController extends BaseController
{

    /**
     * Initialize dimension resource controller.
     *
     * @param type DimensionRepositoryInterface $dimension
     *
     * @return null
     */
    public function __construct(DimensionRepositoryInterface $dimension)
    {
        parent::__construct();
        $this->repository = $dimension;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Question\Repositories\Criteria\DimensionResourceCriteria::class);
    }

    /**
     * Display a list of dimension.
     *
     * @return Response
     */
    public function index(DimensionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Survey\Question\Repositories\Presenter\DimensionPresenter::class)
                ->$function();
        }

        $dimensions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('question::dimension.names'))
            ->view('question::dimension.index', true)
            ->data(compact('dimensions', 'view'))
            ->output();
    }

    /**
     * Display dimension.
     *
     * @param Request $request
     * @param Model   $dimension
     *
     * @return Response
     */
    public function show(DimensionRequest $request, Dimension $dimension)
    {

        if ($dimension->exists) {
            $view = 'question::dimension.show';
        } else {
            $view = 'question::dimension.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('question::dimension.name'))
            ->data(compact('dimension'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new dimension.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DimensionRequest $request)
    {

        $dimension = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('question::dimension.name')) 
            ->view('question::dimension.create', true) 
            ->data(compact('dimension'))
            ->output();
    }

    /**
     * Create new dimension.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DimensionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $dimension                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('question::dimension.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('question/dimension/' . $dimension->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/question/dimension'))
                ->redirect();
        }

    }

    /**
     * Show dimension for editing.
     *
     * @param Request $request
     * @param Model   $dimension
     *
     * @return Response
     */
    public function edit(DimensionRequest $request, Dimension $dimension)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('question::dimension.name'))
            ->view('question::dimension.edit', true)
            ->data(compact('dimension'))
            ->output();
    }

    /**
     * Update the dimension.
     *
     * @param Request $request
     * @param Model   $dimension
     *
     * @return Response
     */
    public function update(DimensionRequest $request, Dimension $dimension)
    {
        try {
            $attributes = $request->all();

            $dimension->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('question::dimension.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('question/dimension/' . $dimension->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('question/dimension/' . $dimension->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the dimension.
     *
     * @param Model   $dimension
     *
     * @return Response
     */
    public function destroy(DimensionRequest $request, Dimension $dimension)
    {
        try {

            $dimension->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('question::dimension.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('question/dimension/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('question/dimension/' . $dimension->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple dimension.
     *
     * @param Model   $dimension
     *
     * @return Response
     */
    public function delete(DimensionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('question::dimension.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('question/dimension'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/question/dimension'))
                ->redirect();
        }

    }

    /**
     * Restore deleted dimensions.
     *
     * @param Model   $dimension
     *
     * @return Response
     */
    public function restore(DimensionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('question::dimension.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/question/dimension'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/question/dimension/'))
                ->redirect();
        }

    }

}
