<?php

namespace Survey\Question\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Survey\Question\Http\Requests\RoptionRequest;
use Survey\Question\Interfaces\RoptionRepositoryInterface;
use Survey\Question\Models\Roption;

/**
 * Resource controller class for roption.
 */
class RoptionResourceController extends BaseController
{

    /**
     * Initialize roption resource controller.
     *
     * @param type RoptionRepositoryInterface $roption
     *
     * @return null
     */
    public function __construct(RoptionRepositoryInterface $roption)
    {
        parent::__construct();
        $this->repository = $roption;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Question\Repositories\Criteria\RoptionResourceCriteria::class);
    }

    /**
     * Display a list of roption.
     *
     * @return Response
     */
    public function index(RoptionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Survey\Question\Repositories\Presenter\RoptionPresenter::class)
                ->$function();
        }

        $roptions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('question::roption.names'))
            ->view('question::roption.index', true)
            ->data(compact('roptions', 'view'))
            ->output();
    }

    /**
     * Display roption.
     *
     * @param Request $request
     * @param Model   $roption
     *
     * @return Response
     */
    public function show(RoptionRequest $request, Roption $roption)
    {

        if ($roption->exists) {
            $view = 'question::roption.show';
        } else {
            $view = 'question::roption.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('question::roption.name'))
            ->data(compact('roption'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new roption.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RoptionRequest $request)
    {

        $roption = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('question::roption.name')) 
            ->view('question::roption.create', true) 
            ->data(compact('roption'))
            ->output();
    }

    /**
     * Create new roption.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RoptionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $roption                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('question::roption.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('question/roption/' . $roption->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/question/roption'))
                ->redirect();
        }

    }

    /**
     * Show roption for editing.
     *
     * @param Request $request
     * @param Model   $roption
     *
     * @return Response
     */
    public function edit(RoptionRequest $request, Roption $roption)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('question::roption.name'))
            ->view('question::roption.edit', true)
            ->data(compact('roption'))
            ->output();
    }

    /**
     * Update the roption.
     *
     * @param Request $request
     * @param Model   $roption
     *
     * @return Response
     */
    public function update(RoptionRequest $request, Roption $roption)
    {
        try {
            $attributes = $request->all();

            $roption->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('question::roption.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('question/roption/' . $roption->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('question/roption/' . $roption->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the roption.
     *
     * @param Model   $roption
     *
     * @return Response
     */
    public function destroy(RoptionRequest $request, Roption $roption)
    {
        try {

            $roption->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('question::roption.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('question/roption/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('question/roption/' . $roption->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple roption.
     *
     * @param Model   $roption
     *
     * @return Response
     */
    public function delete(RoptionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('question::roption.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('question/roption'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/question/roption'))
                ->redirect();
        }

    }

    /**
     * Restore deleted roptions.
     *
     * @param Model   $roption
     *
     * @return Response
     */
    public function restore(RoptionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('question::roption.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/question/roption'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/question/roption/'))
                ->redirect();
        }

    }

}
